// ThreadCPUstate.m
//
// History :
//		10 Sep 92 : created by Max Tardiveau
//		06 Nov 93 : Modified for NextStep 3.1. Now a list object.

#import "ThreadCPUstate.h"

@implementation ThreadCPUstate


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
kern_return_t		error;
thread_state_data_t	states;
unsigned int		stateCount = THREAD_STATE_MAX;
int					i, cellCount;
id					newCell;
char				tmpChar[256];

if ([theCell updater])		// It's a thread state
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

error = thread_get_state((thread_t)[theCell freePointer], 
								THREAD_STATE_FLAVOR_LIST, states, &stateCount);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Threads : error calling thread_get_state().\n");
	return self;
	}

for (i = 0; i < stateCount; i++)
	{
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	sprintf(tmpChar, "State # %i", cellCount);
	[newCell setStringValue:tmpChar];
	[newCell setFreePointer:(void *)states[i]];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	}

[theCell setMasterInstance:self];

return self;
}

////////////////////////////////////////////////////////////////
//

#define MAXRETRIES 3		// Number of times to retry when uninterruptible

- doUpdateForCell:theCell
{
kern_return_t						error;
thread_state_data_t					registers, userRegisters;
unsigned int						regCount = THREAD_STATE_MAX;
unsigned int						userCount = THREAD_STATE_MAX;
char								tmpChar[15];
int									i;
struct thread_basic_info			basicInfo;
unsigned int						basicInfo_count = THREAD_BASIC_INFO_COUNT;

[theCell setUpdater:self];

// Can't examin own thread
if ((thread_t)[theCell freePointer] == thread_self())
	{
	for (i = 0; i < 8; i++)
		[[dataMatrix cellAt:i :0] setStringValue:""];
	for (i = 0; i < 7; i++)
		[[addressMatrix cellAt:i :0] setStringValue:""];
	[StackPointer setStringValue:""];
	[ProgramCounter setStringValue:""];
	[TraceMode setState:0];
	[SuperMode setState:0];
	[IntMask setStringValue:""];
	[Extend setState:0];
	[Negative setState:0];
	[Zero setState:0];
	[Overflow setState:0];
	[Carry setState:0];
	[UserRegister setStringValue:"By Jove"];
	[errorText setStringValue:"You can't examin your own thread."];
	[errorPanel makeKeyAndOrderFront:self];
	return self;
	}

// Can't examin uninterruptible thread
for (i = 0; i < MAXRETRIES; i++)
	{
	error = thread_info((thread_t)[theCell freePointer], THREAD_BASIC_INFO, 
								(thread_info_t)&basicInfo, &basicInfo_count);
	if (error != KERN_SUCCESS)
		{
		[MyMaster addMessage:"ThreadCPUstate : error calling thread_info.\n"];
		return self;
		}
	if (basicInfo.run_state == TH_STATE_UNINTERRUPTIBLE)
		usleep(10000);
	else
		break;
	}

if (basicInfo.run_state == TH_STATE_UNINTERRUPTIBLE)
	{
	for (i = 0; i < 8; i++)
		[[dataMatrix cellAt:i :0] setStringValue:""];
	for (i = 0; i < 7; i++)
		[[addressMatrix cellAt:i :0] setStringValue:""];
	[StackPointer setStringValue:""];
	[ProgramCounter setStringValue:""];
	[TraceMode setState:0];
	[SuperMode setState:0];
	[IntMask setStringValue:""];
	[Extend setState:0];
	[Negative setState:0];
	[Zero setState:0];
	[Overflow setState:0];
	[Carry setState:0];
	[UserRegister setStringValue:"By Jove"];
	[errorText setStringValue:"You can't examin an uninterruptible thread."];
	[errorPanel makeKeyAndOrderFront:self];
	return self;
	}

// Everything OK, get a list of valid flavors
error = thread_get_state((thread_t)[theCell freePointer], 
					THREAD_STATE_FLAVOR_LIST, registers, &regCount);
if (error != KERN_SUCCESS)
	{
	if (error == MIG_ARRAY_TOO_LARGE)
		fprintf(stderr,
			"ThreadCPUstate : array too large in thread_get_state(1).\n");
	else if (error == KERN_INVALID_ARGUMENT)
		fprintf(stderr,
			"ThreadCPUstate : invalid arg in thread_get_state(1).\n");
	else
		fprintf(stderr,
			"ThreadCPUstate : error in thread_get_state(1).\n");
		
	for (i = 0; i < 8; i++)
		[[dataMatrix cellAt:i :0] setStringValue:"Error"];
	for (i = 0; i < 7; i++)
		[[addressMatrix cellAt:i :0] setStringValue:"Error"];
	[StackPointer setStringValue:"Error"];
	[ProgramCounter setStringValue:"Error"];
	[TraceMode setState:0];
	[SuperMode setState:0];
	[IntMask setStringValue:"Error"];
	[Extend setState:0];
	[Negative setState:0];
	[Zero setState:0];
	[Overflow setState:0];
	[Carry setState:0];	
	}
else
	{
	for (i = 0; i < 8; i++)
		{
		sprintf(tmpChar, "%0#.8x", registers.dreg[i]);
		[[dataMatrix cellAt:i :0] setStringValue:tmpChar];
		}
	for (i = 0; i < 7; i++)
		{
		sprintf(tmpChar, "%0#.8x", registers.areg[i]);
		[[addressMatrix cellAt:i :0] setStringValue:tmpChar];
		}
	sprintf(tmpChar, "%0#.8x", registers.areg[7]);
	[StackPointer setStringValue:tmpChar];
	sprintf(tmpChar, "%0#.8x", registers.pc);
	[ProgramCounter setStringValue:tmpChar];
	[TraceMode setState:(registers.sr & 0x8000)];
	[SuperMode setState:(registers.sr & 0x2000)];
	[IntMask setIntValue:((registers.sr & 0x0700) >> 8)];
	[Extend setState:(registers.sr & 0x0010)];
	[Negative setState:(registers.sr & 0x0008)];
	[Zero setState:(registers.sr & 0x0004)];
	[Overflow setState:(registers.sr & 0x0002)];
	[Carry setState:(registers.sr & 0x0001)];
	}

error = thread_get_state((thread_t)[theCell freePointer], 
			NeXT_THREAD_STATE_USER_REG, (int *) &userRegisters, &userCount);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "ThreadCPUstate : error calling thread_get_state(2).\n");
	[UserRegister setStringValue:"Error"];
	}
else
	{
	sprintf(tmpChar, "%0#.8x", userRegisters.user_reg);
	[UserRegister setStringValue:tmpChar];
	}


return self;
}

////////////////////////////////////////////////////////////////
//

- errorOK:sender
{
[NXApp stopModal];
[errorPanel performClose:self];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 5;
}

@end
