// VMContents.m
//
// History :
//		10.25.1992 : created by Max Tardiveau

#import "VMContents.h"

@implementation VMContents



////////////////////////////////////////////////////////////////
//

- init
{
[super init];

myFont = [Font newFont:"Ohlfs"  size:11.0];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t		error;
vm_size_t			size;
vm_prot_t			protection, max_protection;
vm_inherit_t		inheritance;
boolean_t			shared;
port_t				object_name;
vm_offset_t			offset;

[theCell setUpdater:self];

currentTask = (vm_task_t)[theCell freePointer];
currentAddress = (vm_address_t)[theCell secondaryPointer];
error = vm_region(currentTask,&currentAddress, &size, &protection, 
				&max_protection, &inheritance, &shared, &object_name, &offset);

if (error != KERN_SUCCESS)
	{
	[MyMaster addMessage:"VMContents : error calling vm_region(2).\n"];
	return self;
	}

if ( ! (protection & VM_PROT_READ))
	{
	[errorPanel makeKeyAndOrderFront:self];
	return NULL;
	}

currentSize = size / vm_page_size;

[pageNumber setIntValue:0];
[maxPageNumber setIntValue:currentSize - 1];
[self showData:self];

return self;
}

////////////////////////////////////////////////////////////////
//

- showData:sender
{
kern_return_t		error;
unsigned char		*data;
unsigned int		data_count;
char				*buffer, *tmpCharPtr;
int					tmpInt;
int					i, j;

[[myScrollView docView] setText:""];

tmpInt = [pageNumber intValue];
error = vm_read(currentTask, currentAddress + (tmpInt * vm_page_size), 
											vm_page_size, &data, &data_count);
if (error != KERN_SUCCESS)
	{
	[MyMaster addMessage:"VMContents : error calling vm_read.\n"];
	return self;
	}

buffer = malloc(vm_page_size * 5);
tmpCharPtr = buffer;

if ([allASCII state])		// All in ASCII
	{
	for (i = 0; i < (vm_page_size/64); i++)
		{
		sprintf(tmpCharPtr, "%4.4i - ", i * 64);
		tmpCharPtr += 7;
		for (j = 0; j < 64; j++)
			{
			if (data[(i*64)+j] >= ' ' && data[(i*64)+j] < '~')
				sprintf(tmpCharPtr++, "%c", data[(i*64)+j]);
			else
				sprintf(tmpCharPtr++, ".");
			}
		sprintf(tmpCharPtr++, "\n");
		}
	}
else					// Hex and ASCII
	{
	for (i = 0; i < (vm_page_size/16); i++)
		{
		sprintf(tmpCharPtr, "%4.4i - ", i * 16);
		tmpCharPtr += 7;
		for (j = 0; j < 16; j++)
			{
			sprintf(tmpCharPtr , "%2.2x ", data[(i*16)+j]);
			tmpCharPtr += 3;
			}
		for (j = 0; j < 16; j++)
			{
			if (data[(i*16)+j] >= ' ' && data[(i*16)+j] < '~')
				sprintf(tmpCharPtr++, "%c", data[(i*16)+j]);
			else
				sprintf(tmpCharPtr++, ".");
			}
		sprintf(tmpCharPtr++, "\n");
		}
	}

[[myScrollView docView] setText:buffer];
[[myScrollView docView] setFont:myFont];

free(buffer);

return self;
}

////////////////////////////////////////////////////////////////
//

- leftButton:sender
{
[pageNumber setIntValue:[pageNumber intValue] - 1];
[self newPage:self];
return self;
}

////////////////////////////////////////////////////////////////
//

- rightButton:sender
{
[pageNumber setIntValue:[pageNumber intValue] + 1];
[self newPage:self];
return self;
}

////////////////////////////////////////////////////////////////
//

- newPage:sender
{
int		tmpInt;
int		noAction;

noAction = NO;
tmpInt = [pageNumber intValue];

if (tmpInt < 0)
	{
	tmpInt = 0;
	noAction = YES;
	}
	
if (tmpInt > currentSize - 1)
	{
	tmpInt = currentSize - 1;
	noAction = YES;
	}
	
[pageNumber setIntValue:tmpInt];
if (sender == pageNumber)
	noAction = NO;

if ( ! noAction)
	[self showData:self];

return self;
}

@end
